/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Fri Oct 25 10:46:15 EDT 2002
    copyright            : (C) 2002 by Gregory Alan Hildstrom
    email                : ghildstr@hotmail.com
 ***************************************************************************/





#include <iostream>
#include <vector>
#include <fstream>
using namespace std;



//read an entire ascii file into memory
//GetMatrixASCIIInt(FileName, NumberOfChannels, HeaderSize)
vector< vector<float> * > * GetMatrixASCIIInt(char *, int, int, vector< vector<char> >&);
void ReadHeaderASCII(ifstream &,int, vector< vector<char> >&);

//consolidates data points by averaging points together to resample
void ISkipAverage(vector<float>* ,//Input Data Vector
					vector<float>* &,//ISkipped Data Vector
					int //iskip value
					);



int main(int argc, char *argv[])
{

	//intput error checking
	//and useage output
	if(argc !=11){
		cout << "argc = " << argc << endl;
		cout << "Invalid Number Of Arguments.\n";
		cout << "Usage: iskipaverage -if \"input file name\" -hs \"# lines in header\" -nc \"# columns in ascii file\" -np \"# points to iskip and average\" -of \"output file name\"\n\n";
		return 0;
	}//end if
	if(strcmp(argv[1],"-if") ||
		strcmp(argv[3], "-hs") ||
		strcmp(argv[5], "-nc") ||
		strcmp(argv[7], "-np") ||
		strcmp(argv[9], "-of")){
		cout << "Invalid Arguments.\n";
		for(int i=1; i<6; i++) cout << "argv[" << i*2-1 << "] " << argv[i*2-1] << endl;
		cout << "Usage: iskipaverage -if \"input file name\" -hs \"# lines in header\" -nc \"# columns in ascii file\" -np \"# points to iskip and average\" -of \"output file name\"\n\n";
		return 0;
	}//end if


	//obligatory output information
	cout << "ISkipAverage Version Nov-6-2002\n";
	cout << "Copyright 2002 Gregory Alan Hildstrom\n\n";


	//information we will need
	//file name
	//header size
	//number of channels
	//number of points to consolidate
	char* filename = argv[2];
	int headersize = atoi(argv[4]);
	int numberofchannels = atoi(argv[6]);
	int numberofpoints = atoi(argv[8]);
	char* outputfile = argv[10];


	//create our storage matrix in memory
	cout << "Reading Input File...";
	cout.flush();
	vector< vector<float> * > * DataMatrix = NULL;
	vector< vector<float> * > * ISkipAveragedMatrix = NULL;
	vector< vector<char> > HeaderLines;
	DataMatrix = GetMatrixASCIIInt(filename, numberofchannels, headersize, HeaderLines);
	cout << "Done\n";

	//cout << "DataMatrix->size() " << DataMatrix->size() << endl;


	//iskip and average every data channel
	cout << "ISkipAveraging Every Data Channel...";
	cout.flush();
	vector<float>* TempChannel = NULL;
	ISkipAveragedMatrix = new vector<vector<float>*>;
	unsigned int channel = 0;
	for(channel=0; channel < DataMatrix->size(); channel++){
		ISkipAverage((*DataMatrix)[channel], TempChannel, numberofpoints);
		ISkipAveragedMatrix->insert(ISkipAveragedMatrix->end(), TempChannel);
		TempChannel = NULL;
	}//end for
	cout << "Done\n";


	//output channel data into a new ascii file
	cout << "Outputting ISkipAveraged Data To Output File...";
	cout.flush();
	float datapt = 0;
	ofstream outfile;
	outfile.open(outputfile, ios::out);
	//output header again
	for(vector< vector<char> >::iterator v1=HeaderLines.begin(); v1 != HeaderLines.end(); v1++){
		for(vector<char>::iterator v2 = (*v1).begin(); v2 != (*v1).end(); v2++){
			outfile << (*v2);
		}//end for
		outfile << endl;//endl already stored in the header lines
	}//end for
	//output iskipped data
	for(unsigned int datapoint=0; datapoint < (*ISkipAveragedMatrix)[0]->size()-1; datapoint++){
		for(channel=0; channel < ISkipAveragedMatrix->size(); channel++){
				TempChannel = (*ISkipAveragedMatrix)[channel];
				datapt = (*TempChannel)[datapoint];
				outfile << datapt << '\t';
		}//end for
		outfile << endl;
	}//end for
	outfile.close();
	cout << "Done\n\n";


	delete ISkipAveragedMatrix;
	delete DataMatrix;

  return 0;
}
















//this implementation uses binary mode for speedier access and
//multiple char delimiters
//this function imports the entire data file to elimiate ascii
//data input redundancies
vector< vector<float> * > * GetMatrixASCIIInt(char * FilePathName, int NumChannels, int Header, vector< vector<char> >& HeaderLines)
{

	vector< vector<float> * > * DataMatrix;
	DataMatrix = new vector< vector<float> * >;
	for(int c=0; c<NumChannels; c++)
		DataMatrix->insert(DataMatrix->end(), new vector<float>);


	ifstream INFILE; //create input file object
	INFILE.open(FilePathName, ios::in | ios::binary);

	if (INFILE.is_open())
	{//if the file that is named is open...
	
		char* asciidata = new char[50];
		//for(int f = 0; f < 50; f++) asciidata[f] = '\0';
		
		float DataPt;
		char DelimiterChar = '\t';//set the default to tab delimited

		char* ignore = new char;
		*ignore = '*';
		
		int CurrentNum; //stores the index of the characters read in
		ReadHeaderASCII(INFILE, Header, HeaderLines); //skips the header
		//cout << "header check \n";
		//for(int i=0; i<HeaderLines->size(); i++)
			//cout << (*HeaderLines)[i] << endl;
		//cout << endl;

		int dc=0;
		while (!INFILE.eof())
		{
			
			for(dc=0; dc<NumChannels; dc++)//skip to the desired channel
			{
				CurrentNum = 0;
				*ignore = '*';
				while(*ignore != DelimiterChar && *ignore != '\n' &&
							!INFILE.eof() && CurrentNum <50){//read in the data point && CurrentNum < 50
					INFILE.read(ignore,1);
					asciidata[CurrentNum] = *ignore;
					CurrentNum++;
				}//end while

				asciidata[CurrentNum-1] = '\0';
				//cout << "asciidata " << asciidata << endl;
			    			
				if(!INFILE.eof()){
					//converts the data point to a float
					DataPt = atof(asciidata);
						
					//gets the desired data for the channel
					//no need to swap the low byte first data
					//put number into data matrix as a float
					//if(!INFILE.eof())

					(*DataMatrix)[dc]->insert((*DataMatrix)[dc]->end(),DataPt);
				}//end if
			    			
			}//end for
						
		}//end while !INFILE.EOF

		asciidata;
		ignore;

		INFILE.close();
		return DataMatrix;
	
	}//if input file is open
	
	else
	{
		cout<<"\a";//sound alarm
		cout<<"Error: Input file "<< FilePathName
				<< endl << " Not Found" <<endl<<endl;

		return NULL;
	}//not a valid input file
	
}//end of ascii data matrix input



void ReadHeaderASCII(ifstream & INFILE,
			int Header, vector< vector<char> >& HeaderLines)
//INFILE must be open already
{
	int headerlinesize = 0;
	char* temp = new char;

	vector<char> * tempvector;

	for(int i=0; i<Header && !INFILE.eof(); i++){
		*temp = '*';
		tempvector = new vector<char>;
		while(*temp != '\n' && !INFILE.eof()){
			INFILE.read(temp,1);
			if(!INFILE.eof())
				tempvector->insert(tempvector->end(),*temp);
		}//end while
		HeaderLines.insert(HeaderLines.end(), *tempvector);
		delete tempvector;
	}//end for

}//end skip ascii header







//////////////////////////////////
//ISkipAverage Function
//--------------
//This function keeps every i-th point
//from the original data and averages
//it with the points that are skipped
//////////////////////////////////
void ISkipAverage(vector<float>* DataVector,//Input Data Vector
					vector<float>* &ISkipVector,//ISkipped Data Vector
					int I//iskip value
					){
	if(I >= 1 && DataVector != NULL){
	
		if(ISkipVector != NULL){
			delete ISkipVector;
			ISkipVector = NULL;
		}//end if
		if(I >= DataVector->size()){
			cout << "Error: ISkip Value Is Greater or Equal To Data Size.\n";
			return;
		}//end if
		
		ISkipVector = new vector<float>;
		
		float TempSum;
		unsigned int j=0;
		for(unsigned int i=0; i <= DataVector->size()-I+1; i+=I){
			TempSum = 0;
			for(j=0; j < I; j++)
				TempSum += (*DataVector)[i+j];
			TempSum /= I;
			ISkipVector->insert(ISkipVector->end(),TempSum);
		}//end for
	
	
	}//end if
	else{
		cout << "ISkipAverage: Errors!\n";
	}//end else
}//end iskipaverage














